/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.collections.graph;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

public class TreeUtil {

    /**
     * Maximum depth of the directed graph, in other words the depth of the deepest node.
     * 1 if tree has only root, 2 if tree root has one or more direct children (which does not have any) and so on.
     * <p/>
     * NOTE! Works on acyclic directed graphs only! If grapth has cycles, this will hang.
     *
     * @param <E>
     * @param tree
     * @return maximum depth of the directed graph
     */
    public static <E> int maxDepth(final Node<E> tree) {
        int max = 0;
        for (final Node<E> child : tree.getChildren()) {
            int current = maxDepth(child);
            if (current > max) max = current;
        }
        return 1 + max;
    }

    /**
     * Total count of nodes in graph.
     * <p/>
     * NOTE! Works on acyclic directed graphs only! If grapth has cycles, this will hang.
     *
     * @param <E>
     * @param tree
     * @return total number of nodes in graph
     */
    public static <E> int count(final Node<E> tree) {
        int res = 1;
        for (final Node<E> child : tree.getChildren()) {
            res += count(child);
        }
        return res;
    }

    /**
     * Apply visitor v for each node of the graph with root node
     *
     * @param <E>
     * @param node graph (or subgrapth) root
     * @param v    visitor to apply
     */
    public static <E> void forAll(final Node<E> node,
                                  final NodeVisitor<E> v) {
        v.visit(node);
        for (Node<E> e : node.getChildren()) forAll(e, v);
    }

    /**
     * Pretty-print graph
     */
    public static <E> void debugDump(final String prefix, final Node<E> node, final PrintStream out) {
        out.print(prefix);
        out.println(node.getValue());
        final String newPrefix = "\t" + prefix;
        for (final Node<E> n : node.getChildren()) debugDump(newPrefix, n, out);
    }

    /**
     * Return a list of graph leaves. Leaf node is a node that does not have any children.
     * <p/>
     * NOTE! Works on acyclic directed graphs only! If grapth has cycles, this will hang.
     *
     * @return list of grapth leaves
     */
    public static <E> List<Node<E>> extractLeaves(final Node<E> root) {
        final List<Node<E>> res = new ArrayList<Node<E>>();
        extractLeavesImpl(res, root);
        return res;
    }

    private static <E> void extractLeavesImpl(final List<Node<E>> accumulator, final Node<E> node) {
        if (!node.hasChildren()) {
            accumulator.add(node);
        } else {
            for (final Node<E> child : node.getChildren())
                extractLeavesImpl(accumulator, child);
        }
    }
	
}
