/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.collections.graph;

import junit.framework.Assert;

import org.builtu.collections.graph.NodeImpl;
import org.builtu.testing.BasicTestCase;
import org.junit.Before;
import org.junit.Test;


public class TestNodeImplTwo {
	
	private NodeImpl<Long> root, child;
	
	@Before public void createRoot() {
		root = new NodeImpl<Long>("");
		child = new NodeImpl<Long>("child");
	}
	
	@Test public void testAddChild() {
		root.addChild(child);
		assertOne(root, child);
	}

	@Test public void testRemoveChild() {
		final NodeImpl<Long> child2 = new NodeImpl<Long>("child2");
		root.addChild(child2);
		root.addChild(child);
		root.removeChild(child2);
		assertOne(root, child);
	}

	@Test public void testRemoveChildRemovesFRomBothParentAndChild() {
		root.addChild(child);
		root.removeChild(child);
		Assert.assertFalse(root.hasChildren());
		Assert.assertFalse(child.hasParents());
	}
	
	@Test public void testRemoveNonChildReturnsFalse() {
		Assert.assertFalse(root.removeChild(child));
	}
	
	@Test public void testCantAddMyselfAsChild() {
		try {
			root.addChild(root);
		} catch (final IllegalArgumentException e) {}
	}
	

	@Test public void testCannotAddTwoChildrenWithSameId() {
		final NodeImpl<Long> child2 = new NodeImpl<Long>("child");
		root.addChild(child);
		try {
			root.addChild(child2);
			Assert.fail();
		} catch (final IllegalArgumentException e) {}
		assertOne(root, child);
	}
	
	@Test public void testCannotModifyChildren() {
		try {
			root.getChildren().add(child);
		} catch (final UnsupportedOperationException e) {}
	}
	
	@Test public void testCannotModifyParents() {
		try {
			root.getParents().add(child);
		} catch (final UnsupportedOperationException e) {}
	}
	
	@Test public void getNullPath() {
		try {
			root.path(null);
		} catch (final NullPointerException e) {}
	}

	@Test public void getEmptyPath() {
		Assert.assertSame(null, root.path(""));
	}

	@Test public void getSlashedPath1() {
		try {
			root.path("/aaa");
		} catch (final IllegalArgumentException e) {}
	}

	@Test public void getSlashedPath2() {
		try {
			root.path("aaa/");
		} catch (final IllegalArgumentException e) {}
	}
	
	@Test public void getNullChild() {
		try {
			root.get(null);
		} catch (final NullPointerException e) {}
	}

	@Test public void getEmptyChild() {
		Assert.assertSame(null, root.get(""));
	}
	
	@Test public void getSlashedChild1() {
		try {
			root.get("/aaa");
		} catch (final IllegalArgumentException e) {}
	}

	@Test public void getSlashedChild2() {
		try {
			root.get("aaa/");
		} catch (final IllegalArgumentException e) {}
	}

	@Test public void getSlashedChild3() {
		try {
			root.get("aaa/bbbb");
		} catch (final IllegalArgumentException e) {}
	}
	
	@Test public void testPath() {
		root.addChild(child);
		Assert.assertEquals("child", child.getFirstPath());
		final NodeImpl<Long> child2 = new NodeImpl<Long>("child2");
		final NodeImpl<Long> child3 = new NodeImpl<Long>("child3");
		root.addChild(child2);
		child.addChild(child3);
		child2.addChild(child3);
		Assert.assertEquals("child/child3", child3.getFirstPath());
	}
	
	@Test public void testAllPaths() {
		root.addChild(child);
		final NodeImpl<Long> child2 = new NodeImpl<Long>("child2");
		final NodeImpl<Long> child3 = new NodeImpl<Long>("child3");
		root.addChild(child2);
		root.addChild(child3);
		child.addChild(child3);
		
		
		
		Assert.assertTrue(BasicTestCase.collectionContainsExactly(child.getAllPaths(), "child"));
		Assert.assertTrue(BasicTestCase.collectionContainsExactly(child2.getAllPaths(), "child2"));
		Assert.assertTrue(BasicTestCase.collectionContainsExactly(child3.getAllPaths(), "child3", "child/child3"));
	}
	
	private void assertOne(final NodeImpl<Long> root, final NodeImpl<Long> child) {
		Assert.assertEquals(1, root.getChildren().size());
		Assert.assertSame(child, root.getChildren().get(0));
		Assert.assertEquals(1, child.getParents().size());
		Assert.assertSame(root, child.getParents().get(0));
		
		Assert.assertSame(child, root.get("child"));
		Assert.assertSame(child, root.getChild(0));
		Assert.assertSame(child, root.path("child"));
		
		Assert.assertSame(root, child.getParent(0));
	}
	

}
