/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.config.module;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.builtu.util.commutil.Util;


/**
 * Configures and reconfigures modules using Config.
 * 
 * The preferred pattern of usage is following:
 * <code><pre>
 * class B implements Configurable {
 * 		private String attribute;
 * 
 * 		public MyClass() {
 * 			ConfigUtil.configure("mymodule", this);
 * 		}
 * 
 * 		public void configure(final String moduleName, final Config config) {
 * 			attribute = config.getString(moduleName+".some.path.to.attribute");
 * 		}
 * }
 * </pre></code>
 * If all configurable classes follow same pattern, then at any time in future you can call
 * <code>
 * ConfigUtil.reload("mymodule");
 * </code>
 * and have all your classes reconfigured.
 * If you have class A whose configuration depends on another class B, which is, in it's turn, Configurable, then you 
 * must call ConfigUtil.configure() on class A in order to configure it. Also, make sure you call configure(B) before
 * configure(A). ConfigUtil remembers the order of the calls and keeps this order when reloading. The pattern here is:
 * <code><pre>
 * class A implements Configurable {
 * 		private final B b;
 * 		private SomeObject object;
 * 
 * 		public MyClass(final B b) {
 * 			this.b = b;
 * 			ConfigUtil.configure("mymodule", this);
 * 		}
 * 
 * 		public void configure(final String moduleName, final Config config) {
 * 			object = new SomeObject(b.getAttribute());
 * 		}
 * }
 * </pre></code>
 * It is also a good idea to create environment class the holds all module configuration like this:
 * <code><pre>
 * class MyModuleEnv implements Configurable {
 * 		.. atttributes
 * 		public MyModuleEnv() {
 * 			ConfigUtil.configure("mymodule", this);
 * 		}
 * 		...attributes getters/setters
 * 
 * 		private static volatile instance;
 * 		
 * 		public static MyModuleEnv getInstance() {
 * 			...
 * 		}
 * }
 * </pre></code>
 * @author borisman
 */
public class ConfigUtil {
	
	private static final Logger logger = Logger.getLogger(ConfigUtil.class);
	
	private static final Map<String, List<WeakReference<Configurable>>> configurables = 
		new LinkedHashMap<String, List<WeakReference<Configurable>>>();

	/**
	 * Configure the configurable object with configuration from module. As a result, method 
	 * <code> Configurable.configure </code> will be called on configurable
	 * @param moduleName name of the module
	 * @param configurable object tot configure
	 */
	public static void configure(final String moduleName, final Configurable configurable) {
		final Config cfg = getConfig(moduleName);
		configurable.configure(moduleName, cfg);
		final boolean registered = register(moduleName, configurable);
		assert registered : "Can't register "+configurable+", probably already registered";
	}
	
	private static synchronized boolean register(final String moduleName, final Configurable configurable) {
		compact();
		List <WeakReference<Configurable>> list = configurables.get(moduleName);
		if (list == null) {
			list = new ArrayList<WeakReference<Configurable>>();
			configurables.put(moduleName, list);
		}
		final int idx = Util.indexOf(list, configurable);
		if (idx>=0) return false; 
		list.add(new WeakReference<Configurable>(configurable));
		return true;
	}
	
	public static synchronized boolean unregister(final String moduleName, final Configurable configurable) {
		final List <WeakReference<Configurable>> list = configurables.get(moduleName);
		if (list==null) return false;
		final int idx = Util.indexOf(list, configurable);
		if (idx<0) return false;
		list.remove(idx);
		return true;
	}

	public static synchronized void compact() {
		for (final String moduleName : configurables.keySet()) {
			final List<WeakReference<Configurable>> list = configurables.get(moduleName);
			final Iterator<WeakReference<Configurable>> it = list.iterator();
			while (it.hasNext()) {
				if (it.next().get()==null) it.remove();
			}
			if (list.isEmpty()) configurables.remove(moduleName);
		}
	}
	
	public static synchronized void reload(final String moduleName) {
		final List<WeakReference<Configurable>> list = configurables.get(moduleName);
		if (list==null) return;
		final Config config = getConfig(moduleName);
		config.reload();
		final Iterator<WeakReference<Configurable>> it = list.iterator();
		while (it.hasNext()) {
			final Configurable c = it.next().get();
			if (c == null)
				it.remove();
			else {
				logger.info("Reconfiguring "+c);
				c.configure(moduleName, config);
			}
		}
		if (list.isEmpty()) configurables.remove(moduleName);
	}

	public static synchronized void reloadAll() {
		for (final String moduleName : configurables.keySet()) {
			reload(moduleName);
		}
	}
	
	private static final Map<String, Config> configs = new HashMap<String, Config>();
    
    public static synchronized Config getConfig(final String moduleName) {
    	final String confName = '/'+moduleName+".xml";
    	Config res = configs.get(moduleName);
    	if (res == null) {
    		res = new Config(confName);
    		configs.put(moduleName, res);
    	}
    	return res;
	}
    
	
}
