/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.fs;


public class Unix {

	/**
	 *  Copy source file or directory to destination.  
	 *  Behaves almost as unix cp command but does not overwrites anything by default (fails instead) unless flag f 
	 *  specified. Also does not copy files "into" directories.
	 *  Default behavior is:
	 *  <ul>
	 *  	<li>if src is file and no dst present copies src as a file to dst</li>
	 *  	<li>if src is directory - fails</li>
	 *  	<li>if dst presents - fails</li>
	 *  </ul>
	 *  Flags can override this behavior. 
	 *  @param flags <table border=1><tr><th>flag</td><th>short name</th><th>effect</th></tr>
	 *                               <tr><td>r</td><td>recurse</td><td>Process src (even if it is directory) 
	 *                                                                 recursively. Has no effect is src is a file.
	 *                                                                 </td></tr>
	 *                               <tr><td>o</td><td>overwrite</td><td>Overwrite dst (or it's contents) if 
	 *                                                                   present. Fails when destination file 
	 *                                                                   has different type than source file (for
	 *                                                                   example, src is file and dst is directory.
	 *                                                                   This mode is useful for example for updating.
	 *                                                                   directory tree with new content.</td></tr>
	 *                               <tr><td>f</td><td>force</td><td>Same as overwrite, but will overwrite dst (or it's 
	 *                                                               contents) even when destination type is different
	 *                                                               than source type. Almost like removing destination
	 *                                                               prior to copying. Not exactly like it, though, 
	 *                                                               because there may be some files left in dst that 
	 *                                                               were not present in src. Note that it is possible
	 *                                                               to use both f and u flags. In that case update flag 
	 *                                                               happens when source and destination types are same
	 *                                                               and force happens when they are not same.</td></tr>
	 *                               <tr><td>s</td><td>sync</td><td>Remove destination files that are not present in 
	 *                                                              source. Not same as removing whole destination 
	 *                                                              prior to copying, because some files may be left 
	 *                                                              intact when using u flag.</td></tr>
	 *                               <tr><td>u</td><td>update</td><td>Overwrite older files only, left newer intact.
	 *                                                                </td></tr>
	 *               </table> 
	 *  
	 * 
	 * @param src source path (file or directory)
	 * @param dst destination path (file or directory)
	 * @throws IOException
	 * @throws IllegalArgumentException <ul><li>when src missing</li>
	 *                                      <li>when dst present and no o,s,f,u flags specified</li>
	 *                                      <li>when dst present, but has wrong type than src and no f,s flags specified</li>
	 *                                  </ul>    
	 */
	/*public static void cp(final String flags, final File src, final File dst) 
		throws IOException, IllegalArgumentException
	{
		throw new FatalError("Not implemented");
	}

	public static void cp_file(final File src, final File dst, final boolean overwrite) throws IOException {
		
		if (!src.exists()) {
			throw new IllegalArgumentException(src + " does not exists");
		}
		if (!src.isFile()) {
			throw new IllegalArgumentException(src + " is not a file");
		}
		if (dst.exists()) {
			if (dst.isDirectory()) {
				throw new IllegalArgumentException(dst + " is a directory");
			} else if (!overwrite) {
				throw new IllegalArgumentException(dst + " exists");
			}
		}
				
		InputStream in = null;
		OutputStream out = null;
		try {
			in = new FileInputStream (src);
			out = new FileOutputStream(dst);
			 
			IOUtil.stream(in, out);
			
		} finally {
			if (out!=null) out.close();
			if (in!=null) in.close();
		}
	}

	public static void cp_dir(final File src, final File dst, final boolean overwrite) 
		throws IOException 
	{
		if (!src.exists()) throw new IllegalArgumentException(src+" does not exists");
		if (!src.isDirectory()) throw new IllegalArgumentException(src+" is not a directory");
		if (dst.exists()) {
			if (!overwrite) throw new IllegalArgumentException(src+" exists");
		} else {
			if (!dst.mkdir()) throw new FatalError("Can't create "+dst);
		}

		cp_contents(src, dst, overwrite);
	}

	/**
	 * Copy contents of the src directory (recursively) to existing dst directory.
	 *
	 * @param overwrite if true overwite any existing files encountered
	 * @throws IllegalArgumentException when either sourceDir or destDir does not exist or not a directory.
	 */
	/*public static void cp_contents(final File src, final File dst, final boolean overwrite) 
		throws IOException 
	{
		if (!src.exists()) throw new IllegalArgumentException(src+" does not exists");
		if (!dst.exists()) throw new IllegalArgumentException(dst+" does not exists");
		if (!src.isDirectory()) throw new IllegalArgumentException(src+" is not a directory");
		if (!dst.isDirectory()) throw new IllegalArgumentException(dst+" is not a directory");

		final File [] contents = src.listFiles();
		if (contents == null) return;
		for (final File source : contents) {
			final File dest = new File(dst, source.getName());
			if (source.isFile()) {
				cp_file(source, dest, overwrite);
			} else if (source.isDirectory()) {
				cp_dir(source, dest, overwrite);
			} else {
				throw new FatalError(source +" is not a file or directory");
			}
		}
	}

	/**
	 * Move source file to dest file
	 * 
	 * @throws IllegalArgumentException when source missing or dest present
	 */
	/*public static void mv_file(final File source, final File dest) throws IllegalArgumentException, IOException {
		mv_file(source, dest, false);
	}
	
	/**
	 * Move source file to dest file
	 * 
	 * @param overwrite if true, overwrite dest (if present)
	 * @throws IllegalArgumentException when source missing or dest present and overwrite==false
	 */
	/*public static void mv_file(final File source, final File dest, final boolean overwrite) 
		throws IOException, IllegalArgumentException
	{
		if (!source.exists()) throw new IllegalArgumentException(source+" missing");
		if (dest.exists()) {
			if (overwrite) {
				if (!dest.delete()) throw new FatalError("Can't remove "+dest);
			} else {
				throw new IllegalArgumentException(dest + " exists");
			}
		}
		if (!source.renameTo(dest)) {
			cp_file(source, dest);
			if (!source.delete()) throw new FatalError("Can't delete "+source);
		}
	}

	public static void mv_contents(final File sourceDir, final File destDir, final boolean overwrite) 
		throws IOException, IllegalArgumentException 
	{
		final File [] contents = sourceDir.listFiles();
		if (contents == null) return;
		for (final File source : contents) {
			final File dest = new File(destDir, source.getName());
			mv(source, dest);
		}
	}
	
	
	/*
	public static Iterable<String> findFiles(final File dir, final String mask) {
		if (!dir.exists()) throw new IllegalArgumentException(dir+" does not exists");
		return new FindIterable(dir, maskToRegex(mask));
	}

	public static Pattern maskToRegex(final String mask) {
		final StringBuilder res = new StringBuilder(mask.length()<<1);
		res.append('^');
		for (int i=0; i<mask.length(); i++) {
			final char c = mask.charAt(i);
			switch (c) {
			case '?': res.append(".?"); break;
			case '*': res.append(".*?"); break;
			case '\\':
			case '.': 
				res.append('\\');
				res.append(c);
				break;
			default: 
				res.append(Character.toLowerCase(c));
			}
		}
		return Pattern.compile(res.toString());
	}
	
	public static void rm_rf(final String path) {
		rm_rf(new File(path));
	}
	
	public static boolean rm_rf(final File path) {
		boolean res = true;
		final File [] list = path.listFiles(); 
		if (list != null) for (final File f : list) {
			if (f.isFile()) {
				if (!f.delete()) res = false;
			} else {
				if (!rm_rf(f)) res = false;
			}
		}
		if (!path.delete()) res = false;
		return res;
	}
	
	public static void mv (final File source, final File dest) throws IOException {
		if (source.isFile()) 
			mv_file (source, dest);
		else
			mv_dir(source, dest);
	}

	public static void mv_dir(final File sourceDir, final File destDir) throws IOException {
		if (!destDir.exists() && !destDir.mkdirs()) throw new FatalError("Can't create "+destDir);
		mv_contents(sourceDir, destDir);
		if (!sourceDir.delete()) throw new FatalError("Can't delete "+sourceDir);
	}
	
	public static void createDirectory(final File dir) {
		if (!dir.exists() && !dir.mkdir()) throw new FatalError("Can't create "+dir);
	}
	
	public static void createDirectories(final File dir) {
		if (!dir.exists() && !dir.mkdirs()) throw new FatalError("Can't create "+dir);
	}
	
	public static void remove(final File path) {
		if (!path.exists()) return;
		if (path.isFile()) {
			if (!path.delete()) throw new FatalError("Can't delete "+path);
		} else {
			if (!rm_rf(path)) throw new FatalError("Can't delete "+path);
		}
	}
	
	public static void cleanDirectory(final File dir) {
		final File[] files = dir.listFiles();
		if (files == null) return;
		for (final File file : files) {
			remove(file);
		}
	}
	
	public static void rename(final File from, final File to) {
		if (!from.renameTo(to)) throw new FatalError("Can't rename "+from+" -> "+to);
	}
	
	/**
	 * Calculate space taken by a file or directory
	 * @param path file or directory
	 * @return size in bytes on disk of a file or all subfiles of the directory
	 */
	/*public static long du_s(final File path) {
		if (!path.exists()) throw new FatalError("Does not exists: "+path);
		long size = path.length();
		final File [] contents = path.listFiles();
		if (contents == null) return size;
		for (final File f : contents) size += du_s(f);
		return size;
	}*/
}
